package dk.nsp.idsas.ws._20230601;

import com.sun.xml.ws.developer.SchemaValidation;
import dk.nsp.idsas.security.IdsasActorBuilder;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.service.BlurringService;
import dk.nsp.idsas.util.FaultCode;
import dk.nsp.idsas.util.IdsasException;
import dk.nsp.idsas.util.SoapFaultGenerator;
import dk.nsp.idsas.ws.IdsasAbstractWS;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.*;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jws.WebService;
import javax.servlet.annotation.WebServlet;
import javax.xml.ws.BindingType;
import java.time.ZonedDateTime;

import static javax.xml.ws.soap.SOAPBinding.SOAP11HTTP_BINDING;

@WebServlet(name="IdsasRegistration", urlPatterns={"/20230601/registration"})
@WebService(serviceName = "IdsasRegistration", wsdlLocation = "wsdl/20230601/idsas-registration.wsdl",
        targetNamespace = "http://sundhedsdatastyrelsen.dk/identitetssloering/2023/06/01/",
        portName = "IdsasRegistrationPort")
@BindingType(value = SOAP11HTTP_BINDING)
@SchemaValidation
public class IdsasRegistrationWS extends IdsasAbstractWS implements IdsasRegistrationPortType {
    private static final Logger logger = Logger.getLogger(IdsasRegistrationWS.class);

    @Autowired
    private IdsasActorBuilder idsasActorBuilder;

    @Autowired
    private BlurringService blurringService;

    /**
     * Opretter en ny for en borger mod en organisation.<br/>
     * Sikkerheden skal være DGWS niveau 3 (System ID-kort)
     */
    @Override
    public String createBlurring(CreateBlurringRequestType createBlurringRequest) {
        logger.trace("Enter create blurring");

        try {
            IdsasActor actor = idsasActorBuilder.buildActor();
            String organizationId = actor.getOrganizationId();
            String organizationIdType = actor.getOrganizationIdType();
            String patientIdType = createBlurringRequest.getPatientId().getClassification().value();
            String patientId = createBlurringRequest.getPatientId().getId();
            ZonedDateTime endDateTime = createBlurringRequest.getEndDateTime().toGregorianCalendar().toZonedDateTime();

            blurringService.createBlurring(organizationId, organizationIdType, patientId, patientIdType, endDateTime);
        } catch (IdsasException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        } catch (Exception e) {
            logger.error("Failed to create blurring", e);
            throw SoapFaultGenerator.generateSOAPFault(false, "Internal server error",
                    FaultCode.CREATE_BLURRING_ERROR.getValue());
        }

        return "OK";
    }

    /**
     * Opretter en ny afdelingssløring.
     * <ul>
     * <li>Der gemmes {id, orgID, orgClassification, CVR (fra SOSI ID-kortet), validFrom, validTo}</li>
     * <li>Understøttede organisationer i createOrgBlurring skal i første omgang begrænses til SOR og SHAK. Hvis der registreres en SOR-kode, skal det valideres at SOR-koden eksisterer via kald til SORES. Findes den ikke, skal funktionen fejle. SHAK koder kontrolleres ikke.</li>
     * <li>Funktionen skal fejle ved genregistrering af en aktiv afdelingssløring.</li>
     * <li>Sikkerheden skal være DGWS niveau 3 (System ID-kort)</li>
     * <li>ValidFrom sættes til 'nu'</li>
     * </ul>
     * @param createOrgBlurring a new {@code CreateOrgBlurring}
     * @return OK hvis alt går godt
     */
    @Override
    public String createOrgBlurring(CreateOrgBlurring createOrgBlurring) {
        // TODO: Implement
        return "OK";
    }

    /**
     * Returnerer listen af registrerede afdelinger hørende til CVR nummeret i SOSI ID-kortet.<br/>
     * Sikkerheden skal være DGWS niveau 3 (System ID-kort)
     */
    @Override
    public OrgBlurringsResponseType listOrgBlurringsForCVR(OrgIdType listOrgBlurringsForCVRRequest) {
        // TODO: Implement
        return new OrgBlurringsResponseType();
    }

    /**
     * Returnerer listen af registrerede afdelinger hørende til CVR nummeret i SOSI ID-kortet.<br/>
     * Sikkerheden skal være DGWS niveau 3 (System ID-kort)
     */
    @Override
    public OrgBlurringsResponseType listAllActiveOrgBlurrings(EmptyType emptyInput) {
        // TODO: Implement
        return null;
    }

    /**
     * Returnerer listen af alle aktive afdelingssløringer. Skal bruges af tjenester, der skal effektuere afdelingssløringer.<br/>
     * Sikkerheden skal være DGWS niveau 3 (System ID-kort)
     */
    @Override
    public String removeOrgBlurring(OrgIdType removeOrgBlurringRequest) {
        // TODO: Implement
        return "OK";
    }
}
